#! /usr/bin/perl

#   	    Perl Module to Control an ePowerSwitch

#   	    	    	by John Walker
#   	            http://www.fourmilab.ch/

#   	      This program is in the public domain.

#   The ePowerSwitch is a TCP/IP compatible power strip with four
#   outlets which can be controlled via HTTP requests to an embedded
#   mini Web server.  For details about the ePowerSwitch, visit the
#   manufacturer's site:
#
#   	http://www.neol.com/
#
#   The current version of this program is always posted at:
#
#   	http://www.fourmilab.ch/webtools/ePowerSwitch/
#
=head1 NAME

ePowerSwitch - Perl module to control ePowerSwitch

=head1 SYNOPSIS

    use ePowerSwitch;
    
    $eps = ePowerSwitch->new($ip_address, $port,
    	    	    	     $password, $socket);
    $eps->login();
    $eps->logout();
    $are_we = $eps->loggedin();
    $ok = $eps->switch($socket_number, $command);
    $eps->describe($filehandle);
    $swname = $eps->switchname();
    $sockname = $eps->socketname($socket_number);
    $on_or_off = $eps->socketstat($socket_number);
    $vmode = $eps->verbose($mode);

=head1 DESCRIPTION

The ePowerSwitch is a TCP/IP compatible power strip with four
outlets which can be controlled via HTTP requests to an embedded
mini Web server.  For details about the ePowerSwitch, see the
manufacturer's site:

B<http://www.neol.com/>

This module provides an object-oriented interface to an
ePowerStrip.  A program can create one or more
B<ePowerSwitch> objects, each bound to a specific
network-accessible switch (or an individual socket
on a switch), and then query status and switch power
on and off with method calls.

=head1 METHODS

=over 5

=item $eps = ePowerSwitch->new($ip_address, $port, $password, $socket);

The C<new> method is the constructor for the
B<ePowerSwitch> class.  It creates a new B<ePowerSwitch>
with IP address C<$ip_address> and port C<$port>.  You
can create an B<ePowerSwitch> object to access all outlets
on the switch by specifying the administration password for
C<$password> and 0 for C<$socket> (or omitting the
C<$socket> argument), or to access a single socket by
supplying the "user" password for that socket and its
socket number (1 through 4) as C<$socket>.  The
constructor returns a new B<ePowerSwitch> object which may
be manipulated through the following methods.

=item $eps->login();

Logs in to to the power switch (either for administrative access
to all sockets or a specific socket, depending on the C<$password>
and C<$socket> given when the object was created with C<new>).
You must log in before using any methods which query the status
of the switch or turn sockets on or off.  Returns 1 if the login
was successful, 0 if the login failed.

=item $eps->logout();

Logs out from the power switch.  Returns 1 if the logout was
successful, 0 if it fails.  You can log in and out as many times
as you wish.

=item $are_we = $eps->loggedin();

Returns 1 if currently logged in to the switch, 0 if
not logged in.

=item $ok = $eps->switch($socket_number, $command);

Sends a command, specified by a single character
C<$command> argument, to socket C<$socket_number>.  If
you created the B<ePowerSwitch> with the administrative
password, C<$socket_number> can be any socket (1
through 4).  If you gave the password for a single
socket in the call to C<new>, the C<$socket_number>
must be the same as the C<$socket> specified
in the C<new> call.

Commands are as follows:

=over 5

=item C<"0">

Turn socket off.

=item C<"1">

Turn socket on.

=item C<t>

Toggle: if socket is off, turn it on; if on, turn it off.

=item C<r>

Reboot: turn socket off, wait 60 seconds, then turn the
socket back on.  After a reboot command is given, the
C<switch> method returns immediately--it does not wait
until the socket is turned back on.

=back

The C<switch> method returns 1 if the command was executed
normally and 0 if an error occurred.

=item $eps->describe($filehandle);

The properties and status of the B<ePowerSwitch> are printed on
the file opened on C<$filehandle> or, if C<$filehandle> is
omitted, C<STDOUT>.  (f you want to print the status on a
filehandle OFILE, pass a reference to it:
C<describe(\*OFILE)>.) The arguments used to create the object
with C<new> are shown, then the login status.  If logged in to
the switch, its name, and the name and on/off status of each of
the sockets is shown if logged in with the administrative
password.  If logged in with the password for a single socket,
only the status of that socket is shown.

=item $swname = $eps->switchname();

Returns a string containing the name of the B<ePowerSwitch>.
You must be logged in to retrieve the switch name; either
the administrative password or the password for a single
socket entitles you to query the name of the switch.

=item $sockname = $eps->socketname($socket_number);

Returns the name of the socket specified by
C<$socket_number> (1 through 4).  You must be logged in;
if logged in with the administrative password, you may
obtain the name of any socket.  If logged in with the password
for a single socket, only the name of that socket may
be retrieved.

=item $on_or_off = $eps->socketstat($socket_number);

Returns 1 if the specified C<$socket_number> is on, 0 if
it is off.  You must be logged in to query the socket
status.  If logged in with the administrative password, you
can query the status of any socket (1 through 4); if logged
in with the password for a specific socket, only that
socket's status may be obtained.

=item $vmode = $eps->verbose($mode);

By default, errors in communicating with the switch simply
return a zero status to the caller.  (Invalid arguments, however,
use C<die> to report the error.)  If you set verbose mode by
calling C<verbose> with a nonzero C<$mode> argument, diagnostic
messages will be printed on standard output for all operations,
successful or not.  The previous setting of verbose mode
is returned, permitting you to save and restore it.

=back

=head1 VERSION

This is B<ePowerSwitch> version 1.0, released on November 8th, 2003.

=head1 AUTHOR

    John Walker
    http://www.fourmilab.ch/

=head1 BUGS

This module requires the LWP ("Library
for the World Wide Web in Perl") module; if it isn't installed
on your system, you can obtain the current version from CPAN.

The term "socket" is used herein exclusively to refer to
the outlets on the power strip into which you plug
stuff, not network sockets, notwithstanding that the
latter are used in the process of controlling the former.

Please report any bugs to bugs@fourmilab.ch.

=head1 SEE ALSO

L<ePowerSwitch> command line utility.

=head1 COPYRIGHT

This program is in the public domain.

=cut

    require 5.6.0;
    use strict;
    
    use LWP;

    package ePowerSwitch;
    
    require Exporter;
    
    our @ISA = qw(Exporter);
    our @EXPORT = qw(new describe login logout switch loggedin switchname socketname socketstat verbose);
    1;
    
    #	Constructor.  Arguments are:
    #
    #	    IP address of ePowerSwitch
    #	    Port number
    #	    Password (device or specific socket)
    #	    Socket number or 0 if device password given (0 = default if omitted)
    
    sub new {
    	my $self = {};
	my ($invocant, $ip, $port, $password, $socket) = @_;
	my $class = ref($invocant) || $invocant;
	if (!(defined $socket)) {
	    $socket = 0;
	}
	bless($self, $class);

    	if (($socket < 0) || ($socket > 4)) {
	    die("ePowerSwitch::new: Socket number ($socket) out of range.  Must be 0-4");
	}

    	#   Initialise instance variables from constructor arguments
    	$self->{ip} = $ip;
	$self->{port} = $port;
	$self->{password} = $password;
	$self->{socket} = $socket;
	
	$self->{url} = "http://$ip:$port";  	# URL to access mini Web server
	$self->{loggedin} = 0;	    	    	# Not currently logged in
	$self->{http} = LWP::UserAgent->new();	# Create LWP HTTP user agent
	$self->{socketstat} = [];     	    	# Create empty socket status array
	$self->{socketname} = [];     	    	# Create empty socket name array
	$self->{switchname} = '';   	    	# Switch name unknown until login
	$self->{verbose} = 0;	    	    	# Default to non-verbose mode
	
	return $self;
    }
    
    #	Our destructor insures we log out if the user doesn't
    #	explicitly do so.
    
    sub DESTROY {
    	my $self = shift;
	
	if ($self->{verbose}) {
    	    print("ePowerSwitch: Destructor invoked\n");
	}
    	if ($self->{loggedin}) {
	    $self->logout();
	}
    }
    
    #	Print properties of ePowerSwitch object
    
    sub describe {
    	my $self = shift;
	my ($outfile) = @_;
	
	if (!(defined $outfile)) {
	    $outfile = \*STDOUT;
	}
	
	print($outfile "IP: $self->{ip}  Port: $self->{port}  " . 
	      "Password: '$self->{password}'" .
	      (($self->{socket} > 0) ? "  Socket: $self->{socket}" : "") . "\n");
	if ($self->{loggedin}) {
	    print($outfile "  Logged in.\n");
	    print($outfile "  Switch name: $self->{switchname}\n");
	    my $i;
	    
	    for ($i = 1; $i <= 4; $i ++) {
	    	if (defined $self->{socketstat}[$i]) {
		    print($outfile "  Socket $i: $self->{socketname}[$i] " .
		    	(($self->{socketstat}[$i]) ? "On" : "Off") . "\n");
		}
	    }
	} else {
	    print($outfile "  Not logged in.\n");
	}
    }
    
    #	Login to the switch
    
    sub login {
    	my $self = shift;
	
    	my ($content, $status, $ok) = $self->postCommand(
    	    "$self->{url}/elogin.html",
	    [ 'pwd' => $self->{password} ] );

	if ($ok && ($status =~ m/^200\s/)) {
    	    if ($self->{verbose}) {
    	    	print("ePowerSwitch: Login OK.\n");
    	    }
    	    $self->parseReply($content);
	    $self->{loggedin} = 1;
	    return 1;
	} else {	
	    if ($self->{verbose}) {
    	    	print("ePowerswitch: Login error: OK = $ok\nStatus = $status\nContent = $content\n");
	    }
	    return 0;
	}
    }
    
    #	Logout from the switch
    
    sub logout {
    	my $self = shift;
	
    	my ($content, $status, $ok) = $self->postCommand(
    	    "$self->{url}/elogin.html",
	    [ 'X' => '   Exit   ' ] );

	if ($ok && ($status =~ m/^200\s/)) {
	    if ($self->{verbose}) {
    	    	print("ePowerSwitch: Logout OK.\n");
	    }
	    $self->{loggedin} = 0;
	    $self->{socketstat} = [];      	# Clear socket status array
	    $self->{socketname} = [];        	# Clear socket name array
	    $self->{switchname} = '';       	# Switch name unknown unless logged in
	    return 1;
	} else {	
	    if ($self->{verbose}) {
    	    	print("ePowerSwitch: Logout error: OK = $ok\nStatus = $status\nContent = $content\n");
	    }
	    return 0;
	}
    }
    
    #	Send a port control command to the switch.  Arguments are:
    #
    #	    Socket number
    #	    Command:
    #	    	    	'0' 	Turn off
    #	    	    	'1' 	Turn on
    #	    	    	't' 	Toggle (if on, turn off; if off, turn on)
    #	    	    	'r' 	Reboot (turn off, then turn back on 60 seconds later)
    
    sub switch {
    	my $self = shift;
	
	my ($socket, $command) = @_;
	
	#   Construct the pseudo-document name used to send the
	#   control message based upon the socket number given when
	#   we were instantiated.  If we have total device control,
	#   use the global pseudo-document.
	
	my $document = 'econtrol' .
	    	(($self->{socket} == 0) ? '' : "_$self->{socket}") .
		'.html';
    	if (($socket < 1) || ($socket > 4)) {
	    die("ePowerSwitch::switch: Socket number ($socket) out of range.  Must be 1-4");
	}
	
	if (($self->{socket} != 0) && ($self->{socket} != $socket)) {
	    die("ePowerSwitch::switch: Socket number ($socket) does not match socket $self->{socket} of object creation");
	}
	
	if ((length($command) != 1) || ($command !~ m/[01tr]/)) {
	    die("ePowerSwitch::switch: Invalid command \"$command\"");
	}
	
    	my ($content, $status, $ok) = $self->postCommand(
    	    "$self->{url}/$document",
	    [ "P$socket" => $command ] );

	if ($ok && ($status =~ m/^200\s/)) {
	    if ($self->{verbose}) {
    	    	print("ePowerSwitch: Command '$command' OK.\n");
	    }
    	    $self->parseReply($content);
	    return 1;
	} else {	
	    if ($self->{verbose}) {
    	    	print("ePowerSwitch: Command '$command' error: OK = $ok\nStatus = $status\nContent = $content\n");
	    }
	    return 0;
	}
    }
    
    #	Return whether currently logged in
    
    sub loggedin {
    	my $self = shift;
	
    	return $self->{loggedin};
    }
    
    
    #	Return name of switch
    
    sub switchname {
    	my $self = shift;
	
    	if (!$self->{loggedin}) {
	    die("ePowerSwitch::switchname: Not logged in");
	}
	return $self->{switchname};
    }
    
    #	Return name of socket on switch
    
    sub socketname {
    	my $self = shift;
	
	my ($socket) = @_;
	
    	if (!$self->{loggedin}) {
	    die("ePowerSwitch::socketname: Not logged in");
	}
	
    	if (($socket < 1) || ($socket > 4)) {
	    die("ePowerSwitch::socketname: Socket number ($socket) out of range.  Must be 1-4");
	}
	
	if (($self->{socket} != 0) && ($self->{socket} != $socket)) {
	    die("ePowerSwitch::socketname: Socket number ($socket) does not match socket $self->{socket} of object creation");
	}
	
	return $self->{socketname}[$socket];
    }
    
    #	Return on/off status of socket on switch
    
    sub socketstat {
    	my $self = shift;
	
	my ($socket) = @_;
	
    	if (!$self->{loggedin}) {
	    die("ePowerSwitch::socketstat: Not logged in");
	}
	
    	if (($socket < 1) || ($socket > 4)) {
	    die("ePowerSwitch::socketstat: Socket number ($socket) out of range.  Must be 1-4");
	}
	
	if (($self->{socket} != 0) && ($self->{socket} != $socket)) {
	    die("ePowerSwitch::socketstat: Socket number ($socket) does not match socket $self->{socket} of object creation");
	}
	
	return $self->{socketstat}[$socket];
    }

    #	Set and/or query verbose mode.  If called with an argument, sets
    #	verbose mode accordingly.  In any case, returns the current
    #	setting of verbose mode.
    
    sub verbose {
    	my $self = shift;
	
	my $v;
	if ($v = shift) {
	    $self->{verbose} = $v;
	}
	if ($self->{verbose}) {
    	    print("ePowerSwitch: Verbose = $self->{verbose}\n");
	}
	return $self->{verbose};
    }
    
    #	Send a command to the switch with HTTP POST
        
    sub postCommand {
    	my $self = shift;
	
	my $reply = $self->{http}->post(@_);
	return ($reply->content, $reply->status_line,
	    	$reply->is_success, $reply);
    }

    #	Parse the HTML reply text from a switch login or command
    
    #	This code is exquisitely sensitive to the format of the HTML
    #	returned from the ePowerSwitch.  If the manufacturer modifies
    #	the HTML returned, this code will probably break.  If this happens,
    #	uncomment the dump of the HTML reply and dig into the parsing
    #	code to fix it accordingly.
    
    sub parseReply {
    	my $self = shift;	
	my ($reply) = @_;
	
	#   Extract name of the ePowerSwitch from the HTML reply
	
	$reply =~ m/<TD>([^<]+)/;
	my $sn = $1;
	$sn =~ s/^\s+//;
	$sn =~ s/\s+$//;
	$self->{switchname} = $sn;
	
	my $i;
	
	#   Walk through the reply and set the socket status array from the
	#   values returned.
	
	for ($i = 1; $i <= 4; $i++) {
	    if ($reply =~ m/type="hidden" name="P$i" value="(\d)"/) {
    	    	$self->{socketstat}[$i] = $1 ? 0 : 1;
	    }
	}
	
	#   Now rip through the document and build a table of
	#   the names assigned to the sockets.
	
	my $s = $reply;
	
	$i = $self->{socket};
	if ($i == 0) {
	    $i++;
	}

	while ($s =~ s/<TD align=right class=O\w>([^<]+)//i) {
	    $sn = $1;
	    $sn =~ s/^\s+//;
	    $sn =~ s/\s+$//;
	    $self->{socketname}[$i++] = $sn;
	}	
    }

